'use strict';
const moment = require('moment');
const Service = require('egg').Service;

// 本地清洗数据的统计业务   通用所有模块
class StatisticsService extends Service {

  async saveBefore(model, data) {
    const { ctx } = this;
    const { dateString } = data;
    const result = await model.findOne({ dateString });
    ctx.logger.info(`查询本地${model.modelName}时间${dateString}`);
    if (result) {
      return true;
    }
    return false;
  }

  async save(model, data, isUpdate) {
    const { ctx } = this;
    const { dateString } = data;
    ctx.logger.info(`执行本地${model.modelName}时间${dateString}`);
    if (isUpdate) {
      const result = await model.findOne({ dateString });
      if (result) {
        delete data._id;
        const updateResult = await model.updateOne({ dateString }, data);
        if (!updateResult) {
          ctx.throw(`${model.modelName}更新数据异常`);
        }
        return updateResult;
      }
    }
    const createResult = await model.create(data);
    if (!createResult) {
      ctx.throw(`${model.modelName}插入数据异常`);
    }
    return createResult;
  }

  async taskCond(baseData) {
    const { ctx } = this;
    // ?
    await this.sign([ 'app_task' ], baseData, async () => {
      try {
        await ctx.service.appBehaviorRecordService.statistics(baseData);
      } catch (e) {
        await this.writeError(baseData, e, 'appBehaviorRecordService');
      }
      try {
        await ctx.service.appExceptionRecordService.statistics(baseData);
      } catch (e) {
        await this.writeError(baseData, e, 'appExceptionRecordService');
      }
      try {
        await ctx.service.appFeedbackRecordService.statistics(baseData);
      } catch (e) {
        await this.writeError(baseData, e, 'appFeedbackRecordService');
      }
    });
    await this.sign([ 'msc_query_record' ], baseData, async () => {
      try {
        await ctx.service.msgQueryRecordService.statistics(baseData);
      } catch (e) {
        await this.writeError(baseData, e, 'msgQueryRecordService');
      }
    });
    await this.sign([ 'msg_center_record' ], baseData, async () => {
      try {
        await ctx.service.msgCenterRecordService2.statistics(baseData);
      } catch (e) {
        await this.writeError(baseData, e, 'msgCenterRecordService2');
      }
    });
    await ctx.service.tBoxAutoTestService.statistics(baseData);// 重新处理
    await this.sign([ 'stats_base_info', 't_vehicle_record' ], baseData, async () => {
      try {
        await ctx.service.statsBaseInfoService2.statistics(baseData);// 重新处理
      } catch (e) {
        await this.writeError(baseData, e, 'statsBaseInfoService2');
      }
    });

    await this.sign([ 'app_task', 'ivi_online_user', 't_rbac_user' ], baseData, async () => {
      try {
        await ctx.service.tRbacUserService.statistics(baseData);// 重新处理
      } catch (e) {
        await this.writeError(baseData, e, 'tRbacUserService');
      }
      try {
        await ctx.service.onlineUserService2.statistics(baseData);// 重新处理
      } catch (e) {
        await this.writeError(baseData, e, 'onlineUserService2');
      }
    });
    await this.sign([ 'driving_behavior_info', 't_vehicle_record' ], baseData, async () => {
      try {
        await ctx.service.tVehicleRecordService2.statistics(baseData);// 重新处理
      } catch (e) {
        await this.writeError(baseData, e, 'tVehicleRecordService2');
      }
      try {
        await ctx.service.tVehicleReportInfoService.statistics(baseData);// 重新处理
      } catch (e) {
        await this.writeError(baseData, e, 'tVehicleReportInfoService');
      }
      try {
        await ctx.service.drivingBehaviorInfoService2.statistics(baseData);// 重新处理
      } catch (e) {
        await this.writeError(baseData, e, 'drivingBehaviorInfoService2');
      }
      try {
        await ctx.service.tFsmHitoryService.statistics(baseData);// 重新处理
      } catch (e) {
        await this.writeError(baseData, e, 'tFsmHitoryService');
      }
    });
    if (baseData.timeRangData.startTime >= ctx.helper.timeMonthSeven) {
      await this.sign([ 'driving_behavior_info', 'ivi_behavior_record' ], baseData, async () => {
        try {
          await ctx.service.iviBehaviorRecordService.statistics(baseData);// 重新处理
        } catch (e) {
          await this.writeError(baseData, e, 'iviBehaviorRecordService');
        }
      });
    }
  }


  async task(startTime, endTime) { // flag  "delay"  "error"
    const { ctx } = this;
    // 更新历史时间段数据 => 修改createBaseData的参数做循环请求即可
    if (process.env.NODE_ENV != 'development') {
      if (this.app.redis) { // 分布式锁 防止重复执行定时任务
        ctx.logger.info('任务进行 分布式判断可以执行');
        const result = await this.app.redis.setnx(`chart_task_${moment().format('YYYY-MM-DD')}`, '1');
        await this.app.redis.expire(`chart_task_${moment().format('YYYY-MM-DD')}`, 10 * 60);
        if (!result) {
          return;
        }
      } else {
        ctx.logger.info('任务进行 分布式判断忽略执行');
        const newVar = await ctx.model.Local.StatisticsModel.findOne({ }).sort({ create_date: 1 });
        if (newVar) {
          return;
        }
        let satrtMoment;
        if (startTime) {
          satrtMoment = moment(startTime);
        }
        if (!satrtMoment) {
          satrtMoment = moment(this.ctx.helper.yesterday());
        }
        const start = satrtMoment.valueOf();
        const eTime = satrtMoment.add(1, 'days').valueOf();
        const baseData = this.createBaseData(start, eTime);
        await this.writeError(baseData, new Error('任务进行 分布式判断忽略执行'), 'redis');
        return;
      }
    }

    let satrtMoment;
    if (startTime) {
      ctx.logger.info('任务进行 使用传入的开始时间');
      satrtMoment = moment(startTime);
    } else {
      ctx.logger.info('任务进行 查询错误统计表是否存在记录');
      const newVar = await ctx.model.Local.StatisticsModel.findOne({ }).sort({ create_date: 1 });
      if (newVar) {
        satrtMoment = moment(newVar.create_date);
        ctx.logger.info('任务进行 错误统计表存在记录，使用最早的一条时间');
      }
      await ctx.model.Local.StatisticsModel.deleteMany();
      ctx.logger.info('任务进行 删除错误统计表所有记录');
    }
    if (!satrtMoment) {
      satrtMoment = moment(this.ctx.helper.yesterday());
      // satrtMoment = moment('2021-08-10');
    }
    endTime = endTime || this.ctx.helper.today();
    ctx.logger.info(`任务进行 开始${satrtMoment.valueOf()} 结束${endTime}`);
    while (satrtMoment.valueOf() < endTime) {
      const start = satrtMoment.valueOf();
      const eTime = satrtMoment.add(1, 'days').valueOf();
      const baseData = this.createBaseData(start, eTime);

      await this.taskCond(baseData);

    }
  }

  async writeError(baseData, e, msg) {
    const { ctx } = this;
    const errData =
        { ...baseData.initData, start_time: new Date(), end_time: new Date(),
          type: 'error',
          msg: `${msg} : 任务出现错误${baseData.initData.dateString}:${e.message}` };
    this.ctx.logger.info(errData);
    this.ctx.logger.info(e);
    await ctx.model.Local.StatisticsModel.create(errData);
  }

  async sign(ids, baseData, callback) {
    const { ctx } = this;
    let flag = false;
    if (ids && ids.length > 0) {
      const result = await ctx.model.TaskRecordModel.find({ _id: { $in: ids } });
      if (result) {
        flag = result.every(item => {
          const time = moment(item.last_task_data).valueOf();
          return time >= baseData.timeRangData.endTime;
        });
      }
    } else {
      flag = true;
    }

    if (flag) {
      // 可以执行
      await callback();
    } else {
      // 不能执行 延迟执行
      const errData =
          { ...baseData.initData, start_time: new Date(), end_time: new Date(),
            type: 'delay',
            msg: `任务出现延时${baseData.initData.dateString}:${ids.join(',')}` };
      this.ctx.logger.info(errData);
      await ctx.model.Local.StatisticsModel.create(errData);
    }
  }

  createBaseData(start, end) {
    const isForceUpdate = false;
    const yesterday = start || this.ctx.helper.yesterday();
    const today = end || this.ctx.helper.today();
    const initData = {
      year: moment(yesterday).year(),
      month: moment(yesterday).month() + 1,
      day: moment(yesterday).date(),
      dateString: moment(yesterday).format('YYYY-MM-DD'),
      create_date: yesterday,
      // start_time: new Date(),
      // end_time: new Date()
    };
    if (yesterday > today) {
      this.ctx.throw('任务开始时间大于结束时间');
    }
    const timeRangData = { startTime: yesterday, endTime: today };// 这边可以做循环处理
    return { timeRangData, initData, isForceUpdate };
  }
}

module.exports = StatisticsService;
