'use strict';

const Service = require('egg').Service;

// 状态机历史数据服务
class TFsmHitoryService4 extends Service {


  async normal({ type, startTime, endTime, vin }) {
    const { ctx } = this;
    if (vin) {
      const agg1 = [
        { $match: { ...ctx.helper.getTimeRangMatch(startTime, endTime, 'start_time'), vin } },
        { $addFields: ctx.helper.getTimeGenProject(type, 'start_time') },
        { $unwind: '$mileage_list' },
        { $group: ctx.helper.getTimeGroup(type,
          {
            danger_drive_cnt: { $sum: { $add: [ '$mileage_list.lka_cnt', '$mileage_list.ldw_cnt', '$mileage_list.fcw_cnt', '$mileage_list.bsd_cnt' ] } },
            safety_air_bag_cnt: { $sum: { $cond: [{ $eq: [ '$mileage_list.safety_air_bag', 1 ] }, 1, 0 ] } }, // 安全气囊弹出次数
            fault_cnt: { $sum: { $cond: [{ $gt: [ '$mileage_list.fault_duration', 0 ] }, 1, 0 ] } }, // 故障行驶次数
            long_drive_cnt: { $sum: { $cond: [{ $gt: [ '$mileage_list.dsm_duration', 0 ] }, 1, 0 ] } }, // 长时行驶次数
            speeding_cnt: { $sum: '$mileage_list.speeding_cnt' }, // 超速行驶次数
            un_tie_sb_cnt: { $sum: { $cond: [{ $gt: [ '$mileage_list.un_tie_sb_duration', 0 ] }, 1, 0 ] } }, // 未系安全带次数
            leave_sw_cnt: { $sum: '$mileage_list. leave_sw_cnt' }, // 双手离开方向盘次数
            acceleration_cnt: { $sum: '$mileage_list.acceleration_cnt' }, // 急加速次数
            decelerate_cnt: { $sum: '$mileage_list.decelerate_cnt' }, // 加减速次数
            turn_cnt: { $sum: '$mileage_list.turn_cnt' }, // 急转弯次数
            clu_cnt: { $sum: '$mileage_list.clu_cnt' }, // 急换道次数
          }),
        },
      ];
      return await ctx.model.DrivingBehaviorInfoModel.aggregateFix(agg1);
    }
    const agg = [
      { $match: ctx.helper.getTimeRangMatch(startTime, endTime) },
      { $group: ctx.helper.getTimeGroup(type,
        {
          danger_drive_cnt: { $sum: '$danger_drive_cnt' },
          safety_air_bag_cnt: { $sum: '$safety_air_bag_cnt' },
          fault_cnt: { $sum: '$fault_cnt' },
          long_drive_cnt: { $sum: '$long_drive_cnt' }, // 长时行驶次数
          speeding_cnt: { $sum: '$speeding_cnt' }, // 超速行驶次数
          un_tie_sb_cnt: { $sum: '$un_tie_sb_cnt' }, // 未系安全带次数
          leave_sw_cnt: { $sum: '$leave_sw_cnt' }, // 双手离开方向盘次数
          acceleration_cnt: { $sum: '$acceleration_cnt' }, // 急加速次数
          decelerate_cnt: { $sum: '$decelerate_cnt' }, // 加减速次数
          turn_cnt: { $sum: '$turn_cnt' }, // 急转弯次数
          clu_cnt: { $sum: '$clu_cnt' }, // 急换道次数
        }),
      },
    ];
    return await ctx.model.Local.TFsmHitoryModel.aggregateFix(agg);

  }

  async fault({ type, startTime, endTime, vin }) {
    const { ctx } = this;
    if (vin) {
      const agg1 = [
        { $match: { ...ctx.helper.getTimeRangMatch(startTime, endTime, 'start_time'), vin } },
        { $addFields: ctx.helper.getTimeGenProject(type, 'start_time') },
        { $unwind: '$mileage_list' },
        { $group: ctx.helper.getTimeGroup(type,
          {
            fault_type1: { $sum: '$mileage_list.engine_common_fault' }, // 故障类型1
            fault_type2: { $sum: '$mileage_list.electrical_common_fault' }, // 故障类型2
            fault_type3: { $sum: '$mileage_list.transmission_common_fault' }, // 故障类型3
            fault_type4: { $sum: '$mileage_list.steering_common_fault' }, // 故障类型4
            fault_type5: { $sum: '$mileage_list.brake_common_fault' }, // 故障类型5
            fault_type6: { $sum: '$mileage_list.drive_common_fault' }, // 故障类型6
            fault_type7: { $sum: '$mileage_list.boc_common_fault' }, // 故障类型7
          }),
        },
      ];
      return await ctx.model.DrivingBehaviorInfoModel.aggregateFix(agg1);
    }
    const agg = [
      { $match: ctx.helper.getTimeRangMatch(startTime, endTime) },
      { $group: ctx.helper.getTimeGroup(type,
        {
          fault_type1: { $sum: '$fault_type1' }, // 故障类型1
          fault_type2: { $sum: '$fault_type2' }, // 故障类型2
          fault_type3: { $sum: '$fault_type3' }, // 故障类型3
          fault_type4: { $sum: '$fault_type4' }, // 故障类型4
          fault_type5: { $sum: '$fault_type5' }, // 故障类型5
          fault_type6: { $sum: '$fault_type6' }, // 故障类型6
          fault_type7: { $sum: '$fault_type7' }, // 故障类型7
        }),
      },
    ];
    return await ctx.model.Local.TFsmHitoryModel.aggregateFix(agg);

  }

  async intelligence({ type, startTime, endTime, vin }) {
    const { ctx } = this;
    if (vin) {
      const agg1 = [
        { $match: { ...ctx.helper.getTimeRangMatch(startTime, endTime, 'start_time'), vin } },
        { $addFields: ctx.helper.getTimeGenProject(type, 'start_time') },
        { $unwind: '$mileage_list' },
        { $group: ctx.helper.getTimeGroup(type,
          {
            v_parking_cnt: { $sum: '$mileage_list.avp_cnt' }, // 代客泊车次数
            acc_cnt: { $sum: '$mileage_list.acc_cnt' }, // ACC次数
          }),
        },
      ];
      return await ctx.model.DrivingBehaviorInfoModel.aggregateFix(agg1);
    }
    const agg = [
      { $match: ctx.helper.getTimeRangMatch(startTime, endTime) },
      { $group: ctx.helper.getTimeGroup(type,
        {
          v_parking_cnt: { $sum: '$v_parking_cnt' }, // 代客泊车次数
          acc_cnt: { $sum: '$acc_cnt' }, // ACC次数
        }),
      },
    ];
    return await ctx.model.Local.TFsmHitoryModel.aggregateFix(agg);


  }

  async intelligenceMix({ type, startTime, endTime, vin }) {
    const { ctx } = this;
    if (vin) {
      const agg1 = [
        { $match: { ...ctx.helper.getTimeRangMatch(startTime, endTime, 'start_time'), vin } },
        { $addFields: ctx.helper.getTimeGenProject(type, 'start_time') },
        { $unwind: '$mileage_list' },
        { $group: ctx.helper.getTimeGroup(type,
          {
            intel: { $sum: { $cond: [{ $eq: [ '$mileage_list.intelligent_driving_status', '1' ] }, 1, 0 ] } }, // 智能驾驶行程数
            pack: { $sum: { $cond: [{ $gt: [ '$mileage_list.avp_cnt', 0 ] }, 1, 0 ] } }, // 代客泊车行程数
            acc: { $sum: { $cond: [{ $gt: [ '$mileage_list.acc_cnt', 0 ] }, 1, 0 ] } }, // 自适应巡航行程数
            mileage_cnt: { $sum: 1 }, // 行程数
          }),
        },
      ];
      return await ctx.model.DrivingBehaviorInfoModel.aggregateFix(agg1);
    }
    const agg = [
      { $match: ctx.helper.getTimeRangMatch(startTime, endTime) },
      { $group: ctx.helper.getTimeGroup(type,
        {
          intel: { $sum: '$intel' }, // 智能行程
          pack: { $sum: '$pack' }, // 智能行程
          acc: { $sum: '$acc' }, // 智能行程
          mileage_cnt: { $sum: '$mileage_cnt' }, // 总行程
        }),
      },
    ];
    return await ctx.model.Local.TFsmHitoryModel.aggregateFix(agg);

  }


  async statistics({ timeRangData, initData, isForceUpdate }) {
    const { ctx } = this;

    const hasData = await ctx.service.statisticsService.saveBefore(ctx.model.Local.TFsmHitoryModel,
      { ...initData });
    if (hasData && !isForceUpdate) {
      return;
    }
    initData.start_time = new Date();
    const result = await this.group(timeRangData);
      ctx.logger.info('任务进行group');
    await ctx.service.statisticsService.save(ctx.model.Local.TFsmHitoryModel,
      { ...initData, ...result }, isForceUpdate);

  }

  async group({ startTime, endTime }) {
    const { ctx } = this;
    const agg = [
      { $match: ctx.helper.getTimeRangMatch(startTime, endTime, 'start_time') },
      { $unwind: '$mileage_list' },
      { $group: { _id: null,
        danger_drive_cnt: { $sum: { $add: [ '$mileage_list.lka_cnt', '$mileage_list.ldw_cnt', '$mileage_list.fcw_cnt', '$mileage_list.bsd_cnt' ] } },
        safety_air_bag_cnt: { $sum: { $cond: [{ $eq: [ '$mileage_list.safety_air_bag', 1 ] }, 1, 0 ] } }, // 安全气囊弹出次数
        fault_cnt: { $sum: { $cond: [{ $gt: [ '$mileage_list.fault_duration', 0 ] }, 1, 0 ] } }, // 故障行驶次数
        long_drive_cnt: { $sum: { $cond: [{ $gt: [ '$mileage_list.dsm_duration', 0 ] }, 1, 0 ] } }, // 长时行驶次数
        speeding_cnt: { $sum: '$mileage_list.speeding_cnt' }, // 超速行驶次数
        un_tie_sb_cnt: { $sum: { $cond: [{ $gt: [ '$mileage_list.un_tie_sb_duration', 0 ] }, 1, 0 ] } }, // 未系安全带次数
        leave_sw_cnt: { $sum: '$mileage_list. leave_sw_cnt' }, // 双手离开方向盘次数
        acceleration_cnt: { $sum: '$mileage_list.acceleration_cnt' }, // 急加速次数
        decelerate_cnt: { $sum: '$mileage_list.decelerate_cnt' }, // 加减速次数
        turn_cnt: { $sum: '$mileage_list.turn_cnt' }, // 急转弯次数
        clu_cnt: { $sum: '$mileage_list.clu_cnt' }, // 急换道次数
        fault_type1: { $sum: '$mileage_list.engine_common_fault' }, // 故障类型1
        fault_type2: { $sum: '$mileage_list.electrical_common_fault' }, // 故障类型2
        fault_type3: { $sum: '$mileage_list.transmission_common_fault' }, // 故障类型3
        fault_type4: { $sum: '$mileage_list.steering_common_fault' }, // 故障类型4
        fault_type5: { $sum: '$mileage_list.brake_common_fault' }, // 故障类型5
        fault_type6: { $sum: '$mileage_list.drive_common_fault' }, // 故障类型6
        fault_type7: { $sum: '$mileage_list.boc_common_fault' }, // 故障类型7
        v_parking_cnt: { $sum: '$mileage_list.avp_cnt' }, // 代客泊车次数
        acc_cnt: { $sum: '$mileage_list.acc_cnt' }, // ACC次数
        // intel: { $sum: { $cond: [{ $eq: [ '$mileage_list.intelligent_driving_status', '1' ] }, 1, 0 ] } }, // 智能驾驶行程数
        intel: { $sum: { $cond: [{ $or: [ { $gt: [ '$mileage_list.avp_cnt', 0 ] }, { $gt: [ '$mileage_list.acc_cnt', 0 ] } ] }, 1, 0 ] } }, // 智能驾驶行程数
        pack: { $sum: { $cond: [{ $gt: [ '$mileage_list.avp_cnt', 0 ] }, 1, 0 ] } }, // 代客泊车行程数
        acc: { $sum: { $cond: [{ $gt: [ '$mileage_list.acc_cnt', 0 ] }, 1, 0 ] } }, // 自适应巡航行程数
        mileage_cnt: { $sum: 1 }, // 行程数
      } },
    ];
    return await ctx.model.DrivingBehaviorInfoModel.aggregateNGroup(agg);
  }


}

module.exports = TFsmHitoryService4;

